//	Jem.java
//
// top level of Java Emulation of the CSIRAC computer
// no GUI here, all provided by the other modules.
//
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.awt.Container;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.event.KeyEvent;
import java.io.PrintStream;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

public class Jem extends JFrame /*implements ActionListener*/ {
  static final long serialVersionUID = 978563412;
  static final String Version = "V0.1";
  private CPU cpu;
  private KB  kb;
  private Display displays;
  private Container pane;
  public final JFileChooser fc = new JFileChooser();
  private BufferedReader inFile12;
  private BufferedReader inFile5;
  private PrintStream    outFile5;
  private PrintStream    traceFile;
  final static String TF = "0123456789+-.)(ijkdfztogpsx      ABCDEFGHIJKLMNOPQRSTUVWXYZl";  // {Teleprinter - Letter shift}
  final static String FF = "b12*4()78#=-&.\t,0s$3'56/x9+     bQWCRKLUIDVAFMGNPJHEBTYSXOZ";  // Flexowriter
  private int punchlsf=0;
  private int printlsf=0;
  private boolean tracing;
  private Properties props;
  private boolean propsModified;
  private File propsFile;
  
  public Jem(boolean trace) {
    loadProperties();
    cpu = new CPU(this, trace);
    kb  = new KB(this);
    displays = new Display(this);
    setSize(800,400);
    Container pane = getContentPane();
    //    createMenus();
    pane.add(kb);
    pack();
    setTitle("CSIRAC Switch Panel");
    setLocation(320, 260);
    tracing = trace;
    if (tracing) {
      try {
	traceFile = new PrintStream(new FileOutputStream("tlog"));
      }
      catch (IOException e) {
	System.err.println("unable to open tlog");
	System.exit(1);
      }
    }
    setVisible(true);
    kb.setVisible(true);
    
  }

  /*  private void createMenus() {
    MenuBar bar = new MenuBar();
    Menu fileMenu = new Menu("File");
    bar.add(fileMenu);
    MenuItem item = new MenuItem("Program", new MenuShortcut(KeyEvent.VK_P, false));
    item.addActionListener(this);
    fileMenu.add(item);
    item = new MenuItem("Data", new MenuShortcut(KeyEvent.VK_D, false));
    item.addActionListener(this);
    fileMenu.add(item);
    item = new MenuItem("Punch", new MenuShortcut(KeyEvent.VK_S, false));
    item.addActionListener(this);
    fileMenu.add(item);
    item = new MenuItem("Exit");
    item.addActionListener(this);
    fileMenu.add(item);
    Menu help = new Menu("Help");
    item = new MenuItem("Help", new MenuShortcut(KeyEvent.VK_H, false));
    item.addActionListener(this);
    help.add(item);
    item = new MenuItem("About", new MenuShortcut(KeyEvent.VK_A, false));
    item.addActionListener(this);
    help.add(item);
    bar.add(help);
    setMenuBar(bar);
  }
  */
  
  private void loadProperties() {
    String os = System.getProperty("os.name");
    String home = System.getProperty("user.home");
    String line;

    props = null;
    System.out.println("OS = '"+os+"'");
    System.out.println("home = "+home);
    if (os.equals("Linux"))
      propsFile = new File(home+"/.jem.conf");
    else
      propsFile = new File(home+"/Documents/Jem.config");
    System.out.println("propsFile = "+propsFile.getPath());
    if (propsFile.exists()) {
      try {
	props = new Properties();
	props.loadFromXML(new FileInputStream(propsFile));
	propsModified = false;
      }
      catch(IOException e) {
	// ignore failure, create default
      }
    }
    if (props == null) {
      props = new Properties();
      props.setProperty("dir", System.getProperty("user.dir"));	// set to current default
      propsModified = true;
    }
  }
  
  public void terminate() {
    if (props != null && propsModified) {
      try {
	props.storeToXML(new FileOutputStream(propsFile), "CSIRAC Emulator Configuration");
      }
      catch (IOException e) {
	// ignore failure
      }
    }
    System.exit(0);
  } 
  
  public void doSwitch(int bank, int num, boolean state) {
    //    System.out.println("switch "+bank+"/"+num+" set to "+state);
    cpu.switchAction(bank, num, state);
  }

  public void doButton(int num) {
    //    System.out.println("button "+num+" pressed");
    cpu.buttonAction(num);
  }

  public void trace(String S, Object ... args) {
    if (tracing)
      traceFile.print("Trace:"+String.format(S, args));
  }
  public void display(int dnum, int indx, int value) {
    displays.show(dnum, indx, value);
  }

  public void display(int dnum, int value) {
    displays.show(dnum, 0, value);
  }

  public void setTank(int off) {
    displays.setTank(off);
  }
  
  public void setLamp(boolean state) {
    kb.setLamp(state);
  }

  private void error(String s) {
    JOptionPane.showMessageDialog(this, s);
  }

  public File getFile(boolean save) {
    int ret;
    String dir = props.getProperty("dir");
    File cd = dir == null ? null : new File(dir);
    fc.setCurrentDirectory(cd);
    if (save)
      ret = fc.showSaveDialog(this);
    else
      ret = fc.showOpenDialog(this);
    if (ret == JFileChooser.APPROVE_OPTION) {
      File f = fc.getSelectedFile();
      File d = fc.getCurrentDirectory();
      if (cd == null || dir != d.getName()) {
	props.setProperty("dir", d.getPath());
	propsModified = true;
      }
      return f;
    }
    return null;
  }
  
  public void attachPunchFile() {
    File f = getFile(true);
    if (f != null) {
      try {
	outFile5 = new PrintStream(f);
      }
      catch (IOException e) {
	error("Unable to open output file: "+f.getName());
        outFile5 = null;
      }
    }
  }
      
  private void displayHelpInfo() {
    JOptionPane.showMessageDialog(this, "no help yet: try later");
  }

  private void displayAboutInfo() {
    String[] msgs = { "Jem is a CSIRAC Emulator. Version: "+Version,
		      "Author: Bill Purvis",
		      "This version issued: 2021/02/14" };
    JOptionPane.showMessageDialog(this, msgs);
  }
  
  public static void main(String[] args) {
    boolean trace = false;
    if (args.length > 0 && args[0].equals("-t")) trace = true;
    Jem jem = new Jem(trace);
  }

}
